/*=======================================================================================================================
                                                                          
        ARTICLE: The Real Effects of FAS 166/167 on Banks Mortgage Approval and Sale Decisions                      
        AUTHORS: Yiwei Dou, Stephen G. Ryan, Biqin Xie                  
        JOURNAL OF ACCOUNTING RESEARCH 
 
		The code below obtains HMDA data and matches them with variables created using data from FR Y-9C reports. 
 
========================================================================================================================*/



/*========================================================================================================
Step 1: import LARS data into sas
========================================================================================================*/

libname sec 'F:\bxie'; run;

/**********import 2014 LARS data into sas ******************/
%macro lars2014(part);
data LARS_2014_&part;
infile """F:\BXIE\LARS_2014_&part..dat""";

input year 1-4 respondent_id $ 5-14
agency $ 15-15 loan_type 16-16 property_type $ 17-17
loan_purpose 18-18 occupancy 19-19 amount 20-24  preapprovals $ 25-25 
action 26-26 MSA $ 27-31 state $ 32-33 county $ 34-36 census_tract $ 37-43 
ethnicity $ 44-44 co_ethnicity $ 45-45 
race1 $ 46-46 race2 $ 47-47 race3 $ 48-48 race4 $ 49-49 race5 $ 50-50   
co_race1 $ 51-51 co_race2 $ 52-52 co_race3 $ 53-53 co_race4 $ 54-54 co_race5 $ 55-55 
sex $ 56-56 co_sex $ 57-57 income $ 58-61 purchaser_type $ 62-62 
denial1 $ 63-63 denial2 $ 64-64 denial3 $ 65-65 rate_spread $ 66-70 hoepa_status $ 71-71 
lien_status $ 72-72 edit_status $ 73-73 sequence_number $ 74-80 
population $ 81-88 minor_population $ 89-94 hud_median_income $ 95-102 
tract_to_msa_income $103-108 num_occup_houses $ 109-116 
num_14family_houses $ 117-124 appli_date_indicator 125-125 
;
run;

proc means data=LARS_2014_&part; run;
proc freq data=LARS_2014_&part; table action; run;
%mend lars2014;

%lars2014(p1);
%lars2014(p2);
%lars2014(p3);

data LARS_2014_p1k; set LARS_2014_p1; 
	keep year respondent_id agency loan_type property_type loan_purpose occupancy amount action rate_spread hoepa_status lien_status ethnicity 
	MSA state county census_tract 
	race1 sex income purchaser_type ; run;
data LARS_2014_p2k; set LARS_2014_p2; 
	keep year respondent_id agency loan_type property_type loan_purpose occupancy amount action rate_spread hoepa_status lien_status ethnicity 
	MSA state county census_tract 
	race1 sex income purchaser_type ; run;
data LARS_2014_p3k; set LARS_2014_p3; 
	keep year respondent_id agency loan_type property_type loan_purpose occupancy amount action rate_spread hoepa_status lien_status ethnicity    
	MSA state county census_tract 
	race1 sex income purchaser_type ; run;
*stack them up;
data sec.LARS_U2014; set LARS_2014_p1k LARS_2014_p2k LARS_2014_p3k; run;


/**********import 2013 LARS data into sas ******************/
%macro lars2013(part);
data LARS_2013_&part;
infile """F:\BXIE\LARS_2013_&part..dat""";

input year 1-4 respondent_id $ 5-14
agency $ 15-15 loan_type 16-16 property_type $ 17-17
loan_purpose 18-18 occupancy 19-19 amount 20-24  preapprovals $ 25-25 
action 26-26 MSA $ 27-31 state $ 32-33 county $ 34-36 census_tract $ 37-43 
ethnicity $ 44-44 co_ethnicity $ 45-45 
race1 $ 46-46 race2 $ 47-47 race3 $ 48-48 race4 $ 49-49 race5 $ 50-50   
co_race1 $ 51-51 co_race2 $ 52-52 co_race3 $ 53-53 co_race4 $ 54-54 co_race5 $ 55-55 
sex $ 56-56 co_sex $ 57-57 income $ 58-61 purchaser_type $ 62-62 
denial1 $ 63-63 denial2 $ 64-64 denial3 $ 65-65 rate_spread $ 66-70 hoepa_status $ 71-71 
lien_status $ 72-72 edit_status $ 73-73 sequence_number $ 74-80 
population $ 81-88 minor_population $ 89-94 hud_median_income $ 95-102 
tract_to_msa_income $103-108 num_occup_houses $ 109-116 
num_14family_houses $ 117-124 appli_date_indicator 125-125 
;
run;

proc means data=LARS_2013_&part; run;
proc freq data=LARS_2013_&part; table action; run;
%mend lars2013;

%lars2013(p1);
%lars2013(p2);
%lars2013(p3);

data LARS_2013_p1k; set LARS_2013_p1; 
	keep year respondent_id agency loan_type property_type loan_purpose occupancy amount action rate_spread hoepa_status lien_status ethnicity 
	MSA state county census_tract 
	race1 sex income purchaser_type ; run;
data LARS_2013_p2k; set LARS_2013_p2; 
	keep year respondent_id agency loan_type property_type loan_purpose occupancy amount action rate_spread hoepa_status lien_status ethnicity 
	MSA state county census_tract 
	race1 sex income purchaser_type ; run;
data LARS_2013_p3k; set LARS_2013_p3; 
	keep year respondent_id agency loan_type property_type loan_purpose occupancy amount action rate_spread hoepa_status lien_status ethnicity 
	MSA state county census_tract   
	race1 sex income purchaser_type ; run;

data sec.LARS_U2013; set LARS_2013_p1k LARS_2013_p2k LARS_2013_p3k; run; 


/**********import 2012 LARS data into sas ******************/
%macro lars2012(part);
data LARS_2012_&part;
infile """F:\BXIE\LARS_2012_&part..dat""";

input year 1-4 respondent_id $ 5-14
agency $ 15-15 loan_type 16-16 property_type $ 17-17
loan_purpose 18-18 occupancy 19-19 amount 20-24  preapprovals $ 25-25 
action 26-26 MSA $ 27-31 state $ 32-33 county $ 34-36 census_tract $ 37-43 
ethnicity $ 44-44 co_ethnicity $ 45-45 
race1 $ 46-46 race2 $ 47-47 race3 $ 48-48 race4 $ 49-49 race5 $ 50-50   
co_race1 $ 51-51 co_race2 $ 52-52 co_race3 $ 53-53 co_race4 $ 54-54 co_race5 $ 55-55 
sex $ 56-56 co_sex $ 57-57 income $ 58-61 purchaser_type $ 62-62 
denial1 $ 63-63 denial2 $ 64-64 denial3 $ 65-65 rate_spread $ 66-70 hoepa_status $ 71-71 
lien_status $ 72-72 edit_status $ 73-73 sequence_number $ 74-80 
population $ 81-88 minor_population $ 89-94 hud_median_income $ 95-102 
tract_to_msa_income $103-108 num_occup_houses $ 109-116 
num_14family_houses $ 117-124 appli_date_indicator 125-125 
;
run;

proc means data=LARS_2012_&part; run;
proc freq data=LARS_2012_&part; table action; run;
%mend lars2012;

%lars2012(p1);
%lars2012(p2);
%lars2012(p3);

data LARS_2012_p1k; set LARS_2012_p1; 
	keep year respondent_id agency loan_type property_type loan_purpose occupancy amount action rate_spread hoepa_status lien_status ethnicity 
	MSA state county census_tract 
	race1 sex income purchaser_type ; run;
data LARS_2012_p2k; set LARS_2012_p2; 
	keep year respondent_id agency loan_type property_type loan_purpose occupancy amount action rate_spread hoepa_status lien_status ethnicity 
	MSA state county census_tract 
	race1 sex income purchaser_type ; run;
data LARS_2012_p3k; set LARS_2012_p3; 
	keep year respondent_id agency loan_type property_type loan_purpose occupancy amount action rate_spread hoepa_status lien_status ethnicity    
	MSA state county census_tract  
	race1 sex income purchaser_type ; run;
*stack them up;
data sec.LARS_U2012; set LARS_2012_p1k LARS_2012_p2k LARS_2012_p3k; run;


/**********import 2011 LARS data into sas ******************/
%macro lars2011(part);
data LARS_2011_&part;
infile """F:\BXIE\LARS_2011_&part..dat""";

input year 1-4 respondent_id $ 5-14
agency $ 15-15 loan_type 16-16 property_type $ 17-17
loan_purpose 18-18 occupancy 19-19 amount 20-24  preapprovals $ 25-25 
action 26-26 MSA $ 27-31 state $ 32-33 county $ 34-36 census_tract $ 37-43 
ethnicity $ 44-44 co_ethnicity $ 45-45 
race1 $ 46-46 race2 $ 47-47 race3 $ 48-48 race4 $ 49-49 race5 $ 50-50   
co_race1 $ 51-51 co_race2 $ 52-52 co_race3 $ 53-53 co_race4 $ 54-54 co_race5 $ 55-55 
sex $ 56-56 co_sex $ 57-57 income $ 58-61 purchaser_type $ 62-62 
denial1 $ 63-63 denial2 $ 64-64 denial3 $ 65-65 rate_spread $ 66-70 hoepa_status $ 71-71 
lien_status $ 72-72 edit_status $ 73-73 sequence_number $ 74-80 
population $ 81-88 minor_population $ 89-94 hud_median_income $ 95-102 
tract_to_msa_income $103-108 num_occup_houses $ 109-116 
num_14family_houses $ 117-124 appli_date_indicator 125-125 
;
run;

proc means data=LARS_2011_&part; run;
proc freq data=LARS_2011_&part; table action; run;
%mend lars2011;

%lars2011(p1);
%lars2011(p2);
%lars2011(p3);

data LARS_2011_p1k; set LARS_2011_p1; 
	keep year respondent_id agency loan_type property_type loan_purpose occupancy amount action rate_spread hoepa_status lien_status ethnicity 
	MSA state county census_tract 
	race1 sex income purchaser_type ; run;
data LARS_2011_p2k; set LARS_2011_p2; 
	keep year respondent_id agency loan_type property_type loan_purpose occupancy amount action rate_spread hoepa_status lien_status ethnicity  
	MSA state county census_tract 
	race1 sex income purchaser_type ; run;
data LARS_2011_p3k; set LARS_2011_p3; 
	keep year respondent_id agency loan_type property_type loan_purpose occupancy amount action rate_spread hoepa_status lien_status ethnicity  
	MSA state county census_tract   
	race1 sex income purchaser_type ; run;
*stack them up;
data sec.LARS_U2011; set LARS_2011_p1k LARS_2011_p2k LARS_2011_p3k; run;


/**********import 2010 LARS data into sas ******************/
%macro lars2010(part);
data LARS_2010_&part;
infile """F:\BXIE\LARS_2010_&part..dat""";

input year 1-4 respondent_id $ 5-14
agency $ 15-15 loan_type 16-16 property_type $ 17-17
loan_purpose 18-18 occupancy 19-19 amount 20-24  preapprovals $ 25-25 
action 26-26 MSA $ 27-31 state $ 32-33 county $ 34-36 census_tract $ 37-43 
ethnicity $ 44-44 co_ethnicity $ 45-45 
race1 $ 46-46 race2 $ 47-47 race3 $ 48-48 race4 $ 49-49 race5 $ 50-50   
co_race1 $ 51-51 co_race2 $ 52-52 co_race3 $ 53-53 co_race4 $ 54-54 co_race5 $ 55-55 
sex $ 56-56 co_sex $ 57-57 income $ 58-61 purchaser_type $ 62-62 
denial1 $ 63-63 denial2 $ 64-64 denial3 $ 65-65 rate_spread $ 66-70 hoepa_status $ 71-71 
lien_status $ 72-72 edit_status $ 73-73 sequence_number $ 74-80 
population $ 81-88 minor_population $ 89-94 hud_median_income $ 95-102 
tract_to_msa_income $103-108 num_occup_houses $ 109-116 
num_14family_houses $ 117-124 appli_date_indicator 125-125 
;
run;

proc means data=LARS_2010_&part; run;
proc freq data=LARS_2010_&part; table action; run;
%mend lars2010;

%lars2010(p1);
%lars2010(p2);
%lars2010(p3);

data LARS_2010_p1k; set LARS_2010_p1; 
	keep year respondent_id agency loan_type property_type loan_purpose occupancy amount action rate_spread hoepa_status lien_status ethnicity 
	MSA state county census_tract 
	race1 sex income purchaser_type ; run;
data LARS_2010_p2k; set LARS_2010_p2; 
	keep year respondent_id agency loan_type property_type loan_purpose occupancy amount action rate_spread hoepa_status lien_status ethnicity 
	MSA state county census_tract 
	race1 sex income purchaser_type ; run;
data LARS_2010_p3k; set LARS_2010_p3; 
	keep year respondent_id agency loan_type property_type loan_purpose occupancy amount action rate_spread hoepa_status lien_status ethnicity  
	MSA state county census_tract   
	race1 sex income purchaser_type ; run;
*stack them up;
data sec.LARS_U2010; set LARS_2010_p1k LARS_2010_p2k LARS_2010_p3k;run; 


/**********import 2005-2009 LARS data into sas ******************/
%macro lars_0409(year);
data LARS_U&year;
infile """F:\BXIE\LARS_U&year..dat""";

input year 1-4 respondent_id $ 5-14
agency $ 15-15 loan_type $ 16-16 loan_purpose $ 17-17 occupancy $ 18-18 amount $ 19-23 
action $ 24-24 MSA $ 25-29 state $ 30-31 county $ 32-34 census_tract $ 35-41 
sex $ 42-42 co_sex $ 43-43 income $ 44-47 purchaser_type $ 48-48 
denial1 $ 49-49 denial2 $ 50-50 denial3 $ 51-51 edit_status $ 52-52 
property_type $ 53-53 preapprovals $ 54-54 ethnicity $ 55-55 co_ethnicity $ 56-56 
race1 $ 57-57 race2 $ 58-58 race3 $ 59-59 race4 $ 60-60 race5 $ 61-61   
co_race1 $ 62-62 co_race2 $ 63-63 co_race3 $ 64-64 co_race4 $ 65-65 co_race5 $ 66-66 
rate_spread $ 67-71 hoepa_status 72-72 lien_status $ 73-73 sequence_number $ 74-80 
;
run;

proc means data=LARS_U&year; run;

proc freq data=LARS_U&year; table action; run;

data sec.LARS_U&year; set LARS_U&year; 
	keep year respondent_id agency loan_type property_type loan_purpose occupancy amount action rate_spread  hoepa_status lien_status ethnicity  
	MSA state county census_tract 
	race1 sex income purchaser_type;
run;
%mend lars_0409;

%lars_0409(2009); 
%lars_0409(2008); 
%lars_0409(2007); 
%lars_0409(2006); 
%lars_0409(2005);


/*========================================================================================================
Merge respondent_id to call report identifier following Dell'Ariccia et al. (2009). 
"We use the identification numbers each of the three agencies assign to the lenders reporting them HMDA data. 
These numbers correspond to the identification codes RSSD9001 for FRS-regulated lenders, RSSD9050 for FDIC-regulated lenders, 
and RSSD9055 for OCC-regulated lenders in the Call Report database." 

Note: there are indeed many instances where the same respondent_id corresponds to different respondent_rssdid when the agency varies, 
whereas each unique respondent_id & agency combination corresponds to only one unique respondent_rssdid. Hence, the matching between 
HMDA and call report identifier should be done on the combination of two dimensions: respondent_id & agency.
========================================================================================================*/


/*Get rssdhh info from sec.call&year._keep into the dataset to help merge with fr y-9c later.*/
/*Restrict to agencies being OCC FDIC or Federal Reserve.*/
/*agency=3 is FDIC, agency=2 is Federal Reserve, agency=1 is OCC*/

%macro getid(year);
data LARS_U&year; set sec.LARS_U&year; run;

proc print data=LARS_U&year(obs=100); run;

data LARS_U&year; set LARS_U&year; 
	 agency1=input(agency, 1.0);
	 drop agency; 
	 rename agency1=agency; run;

data LARS_U&year; set LARS_U&year;
	if agency=1 or agency=2 or agency=3; 
	if substr(respondent_id,1,2)='00'; run;

data LARS_U&year; set LARS_U&year;
	 respondent_id1=input(respondent_id, 10.0);
	 drop respondent_id; 
	 rename respondent_id1=respondent_id; run;

proc sql; create table LARS_U&year._p1 as select a.*, b.* from LARS_U&year as a, sec.call&year._keep as b 
	where a.agency=1 and a.respondent_id=b.RSSD9055; quit; 

proc sql; create table LARS_U&year._p2 as select a.*, b.* from LARS_U&year as a, sec.call&year._keep as b 
	where a.agency=2 and a.respondent_id=b.RSSD9001; quit; 

proc sql; create table LARS_U&year._p3 as select a.*, b.* from LARS_U&year as a, sec.call&year._keep as b 
	where a.agency=3 and a.respondent_id=b.RSSD9050; quit; 

data sec.LARS_U&year._rssdhh; set LARS_U&year._p1 LARS_U&year._p2 LARS_U&year._p3; run; 
%mend getid;

%getid(2010); 
%getid(2009); 
%getid(2008); 
%getid(2007); 
%getid(2006); 
%getid(2005); 


******starting from 2011, agency=9 (Consumer Financial Protection Bureau) also uses RSSD9001 as ID;
%macro getid2(year);
data LARS_U&year; set sec.LARS_U&year; run;

proc print data=LARS_U&year(obs=100); run;

data LARS_U&year; set LARS_U&year; 
	 agency1=input(agency, 1.0);
	 drop agency; 
	 rename agency1=agency; run;

data LARS_U&year; set LARS_U&year;
	if agency=1 or agency=2 or agency=3 or agency=9; 
	if substr(respondent_id,1,2)='00'; run;

data LARS_U&year; set LARS_U&year;
	 respondent_id1=input(respondent_id, 10.0);
	 drop respondent_id; 
	 rename respondent_id1=respondent_id; run;

proc sql; create table LARS_U&year._p1 as select a.*, b.* from LARS_U&year as a, sec.call&year._keep as b 
	where a.agency=1 and a.respondent_id=b.RSSD9055; quit; 

proc sql; create table LARS_U&year._p2 as select a.*, b.* from LARS_U&year as a, sec.call&year._keep as b 
	where (a.agency=2 or a.agency=9) and a.respondent_id=b.RSSD9001; quit; 

proc sql; create table LARS_U&year._p3 as select a.*, b.* from LARS_U&year as a, sec.call&year._keep as b 
	where a.agency=3 and a.respondent_id=b.RSSD9050; quit; 

data sec.LARS_U&year._rssdhh; set LARS_U&year._p1 LARS_U&year._p2 LARS_U&year._p3; run; /*this is the master LARS data for the year, with rssdhh info to help merge with fr y-9c*/
%mend getid2;

%getid2(2014); 
%getid2(2013); 
%getid2(2012); 
%getid2(2011); 

*Add in a couple of bank-year observations which have a positive amount of Consolidated_VIE_Share in Y9C during 2011-2014 but are currently missing from sec.LARS_U&year._rssdhh,
and hence we have to manually find HMDA match from raw HMDA data;
*add Morgan Stanley in 2011;
data LARS_U2011_few; set sec.LARS_U2011; if agency="9" and respondent_id="0002489805"; run; 

data LARS_U2011_few; set LARS_U2011_few; 
	if respondent_id="0002489805" and agency="9" then rssdhh=2162966; run;

data LARS_U2011_few; set LARS_U2011_few; 
	 agency1=input(agency, 1.0);
	 drop agency; 
	 rename agency1=agency; run;

data LARS_U2011_few; set LARS_U2011_few;
	if agency=1 or agency=2 or agency=3 or agency=9; 
	if substr(respondent_id,1,2)='00'; run;

data LARS_U2011_few; set LARS_U2011_few;
	 respondent_id1=input(respondent_id, 10.0);
	 drop respondent_id; run;

data LARS_U2011_few; set LARS_U2011_few;
	 rename respondent_id1=respondent_id; run; 
data sec.LARS_U2011_few; set LARS_U2011_few; 

*Add IMB and Flagstar in 2012 ;
data LARS_U2012_few; set sec.LARS_U2012; if agency="9" and (respondent_id="0003918898" or respondent_id="0000146672"); run; 

data LARS_U2012_few; set LARS_U2012_few; 
	if respondent_id="0003918898" and agency="9" then rssdhh=3923614;
	if respondent_id="0000146672" and agency="9" then rssdhh=3852022; run;

data LARS_U2012_few; set LARS_U2012_few; 
	 agency1=input(agency, 1.0);
	 drop agency; 
	 rename agency1=agency; run;

data LARS_U2012_few; set LARS_U2012_few;
	if agency=1 or agency=2 or agency=3 or agency=9; 
	if substr(respondent_id,1,2)='00'; run;

data LARS_U2012_few; set LARS_U2012_few;
	 respondent_id1=input(respondent_id, 10.0);
	 drop respondent_id; run;

data LARS_U2012_few; set LARS_U2012_few;
	 rename respondent_id1=respondent_id; run; 
data sec.LARS_U2012_few; set LARS_U2012_few; 


*add IMB, Flagstar, and Discover in 2013;
data LARS_U2013_few; set sec.LARS_U2013; 
	if (agency="9" and (respondent_id="0003918898" or respondent_id="0000146672")) or (agency="2" and respondent_id="0004421069"); run; 

data LARS_U2013_few; set LARS_U2013_few; 
	if respondent_id="0003918898" and agency="9" then rssdhh=3923614;
	if respondent_id="0000146672" and agency="9" then rssdhh=3852022; 
	if respondent_id="0004421069" and agency="2" then rssdhh=3846375; run;

data LARS_U2013_few; set LARS_U2013_few; 
	 agency1=input(agency, 1.0);
	 drop agency; 
	 rename agency1=agency; run;

data LARS_U2013_few; set LARS_U2013_few;
	if agency=1 or agency=2 or agency=3 or agency=9; 
	if substr(respondent_id,1,2)='00'; run;

data LARS_U2013_few; set LARS_U2013_few;
	 respondent_id1=input(respondent_id, 10.0);
	 drop respondent_id; run;

data LARS_U2013_few; set LARS_U2013_few;
	 rename respondent_id1=respondent_id; run; 
data sec.LARS_U2013_few; set LARS_U2013_few; 


*add IMB and Flagstar in 2014;
data LARS_U2014_few; set sec.LARS_U2014; if agency="9" and (respondent_id="0003918898" or respondent_id="0000146672"); run; 

data LARS_U2014_few; set LARS_U2014_few; 
	if respondent_id="0003918898" and agency="9" then rssdhh=3923614;
	if respondent_id="0000146672" and agency="9" then rssdhh=3852022; run;

data LARS_U2014_few; set LARS_U2014_few; 
	 agency1=input(agency, 1.0);
	 drop agency; 
	 rename agency1=agency; run;

data LARS_U2014_few; set LARS_U2014_few;
	if agency=1 or agency=2 or agency=3 or agency=9; 
	if substr(respondent_id,1,2)='00'; run;

data LARS_U2014_few; set LARS_U2014_few;
	 respondent_id1=input(respondent_id, 10.0);
	 drop respondent_id; run;

data LARS_U2014_few; set LARS_U2014_few;
	 rename respondent_id1=respondent_id; run; 
data sec.LARS_U2014_few; set LARS_U2014_few; 


/*for 2011-2014, merge in *Add in a couple of bank-year observations which have a positive amount of Consolidated_VIE_Share in Y9C during 2011-2014 but are */
/*currently missing from sec.LARS_U&year._rssdhh,and hence we have to manually find HMDA match from raw HMDA data*/
%macro con(year);
data sec.LARS_U&year._rssdhh; set sec.LARS_U&year._rssdhh sec.LARS_U&year._few; run;
proc contents data=sec.LARS_U&year._rssdhh; run;
%mend con;

%con(2014);
%con(2013);
%con(2012);
%con(2011);




/*========================================================================================================
Step 2: define loan application variables 
/*========================================================================================================*/

*************macro for year 2010-2014. because amount and action are already numeric during 2010-2014, which is different from 2003-2009, so do 2010-2014 separately**********;
%macro create (year);
data LARS_U&year._rssdhh; set sec.LARS_U&year._rssdhh;

	/*restrict to approved or denied applications*/
	if action=1 or action=2 or action=3; 

    if action=1 or action=2 then approval=1;
	if action=3 then approval=0;

	if purchaser_type=. then sold_unaffi=.;
	if purchaser_type~=. and purchaser_type ne 0 and purchaser_type ne 8 then sold_unaffi=1;  
	if purchaser_type~=. and (purchaser_type eq 0 or purchaser_type eq 8) then sold_unaffi=0; 

	if approval=0 then do; sold=.; sold_unaffi=.; end; run;


	income2=input(income, 8.0);
	drop income; 
	rename income2=income;

	if income gt 0;
	if amount gt 0;

	dti=amount/income; 
	label dti="loan to income ratio";

	if race1="1" or race1="2" or race1="3" or race1="4" or race1="5";

	if sex="1" or sex="2";

	if ethnicity="1" or ethnicity="2"; 

	/*define refinancing. 3 is refinancing*/
	if loan_purpose=3 then refinance=1; 
	if loan_purpose=1 or loan_purpose=2 then refinance=0;

	if sex="1" then male=1; else male=0; 
	if race1="3" then b=1; else b=0; 
	if race1="5" then w=1; else w=0; 

	/*combine state and county into FIPS_code to help merge with unemploy data*/
	county5=state || county; 

	if approval~=.; 
	if dti gt 0; 
	if refinance~=.;
	if male ~=.;
	if b~=.;
	if w~=.; 
	if rssdhh~=.;
	run;

data LARS_U&year._rssdhh2; set LARS_U&year._rssdhh2;
	/*prepare for winsorization*/
	dti_99=dti;
	amount_99=amount;
	income_99=income; run;

%mend create;
%create(2014);
%create(2013); 
%create(2012); 
%create(2011); 
%create(2010); 


*************macro for year 2004-2009. Differ from 2010-2014 since 2004-2009 amount and action are character*********;

%macro create (year);
data LARS_U&year._rssdhh; set LARS_U&year._rssdhh;
	/*restrict to approved or denied applications*/
	if action="1" or action="2" or action="3"; 

	 if action="1" or action="2" then approval=1;
	 if action="3" then approval=0;

	if purchaser_type=. then sold_unaffi=.;
	if purchaser_type~=. and purchaser_type ne 0 and purchaser_type ne 8 then sold_unaffi=1;  
	if purchaser_type~=. and (purchaser_type eq 0 or purchaser_type eq 8) then sold_unaffi=0; 

	if income~="NA"; 
	income2=input(income,8.0); 
	drop income;
	rename income2=income;

	 if amount~="NA";
	 amount2 = input(amount,8.0); 
	 drop amount; 
	 rename amount2=amount; 

	if income gt 0;
	if amount gt 0;

	dti=amount/income; 
	label dti="loan to income ratio";

	if race1="1" or race1="2" or race1="3" or race1="4" or race1="5";

	if sex="1" or sex="2";

	if ethnicity="1" or ethnicity="2"; 

	/*define refinancing. 3 is refinancing*/
	if loan_purpose="3" then refinance=1; 
	if loan_purpose="1" or loan_purpose="2" then refinance=0;

	if sex="1" then male=1; else male=0; 
	if race1="3" then b=1; else b=0; 
	if race1="5" then w=1; else w=0; 

	*combine state and county into FIPS_code to help merge with unemploy data;
	county5=state || county;

	if approval~=.; 
	if dti gt 0; 
	if refinance~=.;
	if male ~=.;
	if b~=.;
	if w~=.; 
	if rssdhh~=.;

	run;


data LARS_U&year._rssdhh2; set LARS_U&year._rssdhh2;
	/*prepare for winsorization*/
	dti_99=dti;
	amount_99=amount;
	income_99=income; run;

data LARS_U&year._rssdhh2_june2016; set LARS_U&year._rssdhh2; run;
%mend create;
%create(2009); 
%create(2008);
%create(2007);
%create(2006);
%create(2005);


/*==================================================
get conforming limit data for 2005-2014
/*==================================================*/

%macro limit(yr);
PROC IMPORT OUT= Conforming_&yr  
DATAFILE= "E:\4securitization\sasdata\conforming_loan_limits_2005_2014.xlsx" 
            DBMS=xlsx REPLACE;
     SHEET="GSE_&yr."; 
     GETNAMES=YES;
RUN; *18 obs;
data sec.Conforming_&yr; set Conforming_&yr;
	yr=&yr; 
	keep County_Name FIPS_County_Code FIPS_State_Code State limit yr; run;
proc print data=sec.Conforming_&yr(obs=20); run;
proc contents data=sec.Conforming_&yr; run;
%mend limit;

%limit(2014);
%limit(2013);
%limit(2012);
%limit(2011);
%limit(2010);
%limit(2009);
%limit(2008);
%limit(2007);
%limit(2006);
%limit(2005);

*stack up;
data Conforming_limit_0514; 
	set sec.Conforming_2014 sec.Conforming_2013 sec.Conforming_2012 sec.Conforming_2011 sec.Conforming_2010 sec.Conforming_2009 
	sec.Conforming_2008 sec.Conforming_2007 sec.Conforming_2006 sec.Conforming_2005; run; 

proc contents data=Conforming_limit_0514; run;

data sec.Conforming_limit_0514; set Conforming_limit_0514;
	county5=FIPS_State_Code || FIPS_County_Code; run;



/*create conforming_loan variable after getting conforming limit from sec.conforming_limit_0514;*/
%macro conf(year);
proc sql; create table LARS_U&year._rssdhh3 
	as select a.*, b.limit/1000 as limit from LARS_U&year._rssdhh2 as a 
	left join sec.conforming_limit_0514 as b 
	on a.county5=b.county5 
	and a.year=b.yr; quit;


data LARS_U&year._rssdhh3; set LARS_U&year._rssdhh3;
	if limit gt 0 and amount le limit then conforming_loan=1;
	if limit gt 0 and amount gt limit then conforming_loan=0;
	if limit=. then conforming_loan=.; 

	rename race1=race; 
	rename hoepa_status=hoepa;
	rename lien_status=lien; run;

data sec.LARS_U&year._rssdhh_final_june2016; set LARS_U&year._rssdhh3; run;

%mend conf;

%conf(2014);
%conf(2013);
%conf(2012);
%conf(2011);
%conf(2010);
%conf(2009);
%conf(2008);
%conf(2007);
%conf(2006);
%conf(2005);



/*========================================================================================================
Step 3: create stratified random sampling of contract-level LARS data for 2005-2014 using firm-year-specific optimal sample size. 
set SRS formula so that we choose a t value of 1.95 and a d value of 0.01, which correspond to 95% confidence that the denial rate 
in the stratified random sample differs from the denial rate based on all applications received by a bank in a given year by more 
than 1% with less than 1% frequency. 
Before conducting SRS, first remove obs with missing MSA and impose sample restriction following Dell'Ariccia et al. (2009) (i.e., removing income<=$10k or amount<=$1k).
Number of apps in the final srs sample to quality as a sample bank-yr obs is indicated using _NSIZE_sus_.
========================================================================================================*/

/*macro code when using firm-year-specific optimal sample size regardless of the population size*/
******macro code********;
%macro srs_opti_size_cfd95_sus_june02(year);
data LARS_U&year._rssdhh22; set sec.LARS_U&year._rssdhh_final_june2016; 
	if MSA="NA" then delete;
	/*screening following Dell'Ariccia et al. (2009)*/
	if income le 10 then delete; if amount le 1 then delete; 
run;

/*get bank-year specific mean_sus_approval and sum_sus_app to be used to determine optimal sampling sizes;*/
/*get sum_sus_app, which is total number of unsold + sold  nonrefinancing + refinancing applications received by a bank-yr*/
proc sort data=LARS_U&year._rssdhh22; by rssdhh; run;
proc univariate data=LARS_U&year._rssdhh22 noprint; var amount; 
	by rssdhh; 
	output out=s2_&year n=sum_sus_app; run;

/*get mean_sus_approval which is the approval rate of unsold + sold  nonrefinancing + refinancing applications for a bank-yr*/
proc sort data=LARS_U&year._rssdhh22; by rssdhh; run;
proc univariate data=LARS_U&year._rssdhh22 noprint; var approval; 
	by rssdhh; 
	output out=s3_&year mean=mean_sus_approval; run;

/*get sum_sus_app and mean_sus_approval into LARS_U&year._rssdhh2;*/
proc sql; create table LARS_U&year._rssdhh3 as select a.*, b.sum_sus_app    
	from LARS_U&year._rssdhh22 as a 
	left join s2_&year as b 
	on a.rssdhh=b.rssdhh; quit;

proc sql; create table LARS_U&year._rssdhh33 as select a.*, b.mean_sus_approval    
	from LARS_U&year._rssdhh3 as a 
	left join s3_&year as b 
	on a.rssdhh=b.rssdhh; quit;

/*conduct SRS*/
data LARS_U&year._rssdhh33; set LARS_U&year._rssdhh33;
	srs_n=(1.96*1.96*mean_sus_approval*(1-mean_sus_approval) + 0.01*0.01)/(1.96*1.96*mean_sus_approval*(1-mean_sus_approval)/sum_sus_app + 0.01*0.01); 
	_NSIZE_=round(srs_n); run;

data ss&year; set LARS_U&year._rssdhh33; 
	keep rssdhh _NSIZE_; 
run;

proc means data=ss&year n std mean min p1 p5 p10 p25 median p75 p90 p95 p99 max; var _NSIZE_; run;

proc sort data=LARS_U&year._rssdhh33; by rssdhh; run;
proc sort data=ss&year; by rssdhh; run;

proc surveyselect data = LARS_U&year._rssdhh33 out =LARS_srs_confdc95_&year._sus method = srs SAMPSIZE=ss&year seed = 9876;
strata rssdhh; run; 

data LARS_srs_confdc95_&year._sus; set LARS_srs_confdc95_&year._sus;
	rename _NSIZE_=_NSIZE_sus_; run;

%mend srs_opti_size_cfd95_sus_june02;

%srs_opti_size_cfd95_sus_june02(2014);
%srs_opti_size_cfd95_sus_june02(2013);
%srs_opti_size_cfd95_sus_june02(2012);
%srs_opti_size_cfd95_sus_june02(2011); 
%srs_opti_size_cfd95_sus_june02(2010);
%srs_opti_size_cfd95_sus_june02(2009); 
%srs_opti_size_cfd95_sus_june02(2008); 
%srs_opti_size_cfd95_sus_june02(2007); 
%srs_opti_size_cfd95_sus_june02(2006); 
%srs_opti_size_cfd95_sus_june02(2005);


*convert character vars into numeric;
%macro cv(year);
data LARS_srs_confdc95_&year._sus2; set LARS_srs_confdc95_&year._sus;
	 hoepa2=input(hoepa, 8.0);
	 lien2=input(lien, 8.0);
	 sex2=input(sex, 8.0);
	 property_type2=input(property_type, 8.0);
	 race2=input(race, 8.0);
	 ethnicity2=input(ethnicity, 8.0);

	 drop hoepa lien sex property_type race ethnicity; 

	 rename hoepa2=hoepa; 	
	 rename lien2=lien;
	 rename sex2=sex; 
	 rename property_type2=property_type; 
	 rename race2=race; 
	 rename ethnicity2=ethnicity; run;  
%mend cv;

%cv(2014);
%cv(2013);
%cv(2012);
%cv(2011);
%cv(2010);


%macro cv(year);
data LARS_srs_confdc95_&year._sus2; set LARS_srs_confdc95_&year._sus;
	loan_type2=input(loan_type, 8.0);
	loan_purpose2=input(loan_purpose, 8.0);
	occupancy2=input(occupancy, 8.0);
	action2=input(action, 8.0);
	lien2=input(lien, 8.0);

	drop loan_type loan_purpose occupancy action /*hoepa*/ lien;
	rename loan_type2=loan_type;
	rename loan_purpose2=loan_purpose;
	rename occupancy2=occupancy;
	rename action2=action;
	rename lien2=lien;

	 sex2=input(sex, 8.0);
	 property_type2=input(property_type, 8.0);
	 race2=input(race, 8.0);
	 ethnicity2=input(ethnicity, 8.0);
	 drop sex property_type race ethnicity;
	 rename sex2=sex; 
	 rename property_type2=property_type; 
	 rename race2=race; 
	 rename ethnicity2=ethnicity; run;  
%mend cv;

%cv(2009);
%cv(2008);
%cv(2007);
%cv(2006);
%cv(2005); 


*stack them up;
data sec.LARS_srs_cfd95_0514_sus_june02; set LARS_srs_confdc95_2014_sus2 LARS_srs_confdc95_2013_sus2 LARS_srs_confdc95_2012_sus2 LARS_srs_confdc95_2011_sus2 LARS_srs_confdc95_2010_sus2 
	LARS_srs_confdc95_2009_sus2 LARS_srs_confdc95_2008_sus2 LARS_srs_confdc95_2007_sus2 LARS_srs_confdc95_2006_sus2 LARS_srs_confdc95_2005_sus2 
	; run;  *This is the master SRS dataset for unsold + sold  nonrefinancing + refinancing apps 
	during 2005-2014 with SRS sample size equal to the integer of n = [ ( z2* p * q ) + ME2] / [ ME2+ z2* p * q / N ], where z=1.96 since i require confidence level of 95%, 
	and margin of error ME=1%. number of obs in this srs sample is _NSIZE_sus_;



/*========================================================================================================
Step 4: merge in Y9C variables
========================================================================================================*/

data LARS_h2; set sec.LARS_srs_cfd95_0514_sus_june02;
	keep rssdhh year approval  
	amount income dti male b w conforming_loan per_capita_income unemploy_rate _NSIZE_sus_ 
	MSA county5 action purchaser_type 

	loan_type loan_purpose property_type occupancy sex race ethnicity hoepa lien;

	if year ge 2005;
/*	screening: require _NSIZE_sus_>=50*/
	if _NSIZE_sus_>=50;	
run; 


*merge in Y9C variables;
data ss4; set sec.data_y9c;
	if ta06 gt 500000 and yr ge 2005; 
	keep rid name yr cpi_annual_convertto_06 hlr ta06  
	vieall_exvieall_r secuob_exvieall_r 
	post stress lsec_ta_r lfiloan lploanr2 lcdr ldeposit_rate ch_wsf_rta lroa_crisis_loss2 lroa_ex_crisis_loss2 lt1rcapr2 lsize06 dift1rcapr st_funding  
	abcp; run; 


/*keep only obs in ss4 that have obs in sec.LARS_srs_cfd95_0514_sus_june02*/
data bky1; set LARS_h2;
	keep rssdhh year; run; 
proc sort data=bky1 nodupkey out=bky2; by rssdhh year; run; 

proc sql; create table ss4 
	as select a.* from ss4 as a, bky2 as b 
	where a.rid=b.rssdhh and a.yr=b.year; quit; 

/*winsor Y9C variables in ss4*/
data ss4; set ss4;
	secuob_unwin=secuob_exvieall_r; 
	secuob_r_win99all_byyr=secuob_exvieall_r; run; 

%let var_wins=lsec_ta_r lfiloan lploanr2 lcdr ldeposit_rate ch_wsf_rta lroa_crisis_loss2 lroa_ex_crisis_loss2 lt1rcapr2 lsize06;
%wins(ss4,99,1,yr); *6027;


*take positive secuob_exvieall_r out and winsorize it at 1% and 99% level for the entire sample period;
data b3; set ss4;
	if secuob_exvieall_r gt 0; 
	keep rid yr secuob_exvieall_r;
	run;
proc sort data=b3 out=b4 nodupkey; by rid yr; run;

data b4; set b4;
	secuob_r_win99positive_allyrs=secuob_exvieall_r; run;
	
%wins_bynone(dsetin=b4, dsetout=b5, byvar=none, vars=secuob_r_win99positive_allyrs, type=winsor, pctl=1 99);


/*get secuob_r_win99positive_allyrs*/
proc sql; create table ss4 
	as select a.*, b.secuob_r_win99positive_allyrs 
	from ss4 as a 
	left join b5 as b 
	on a.rid=b.rid and a.yr=b.yr; quit; *6034;

data ss4; set ss4;
	if secuob_r_win99positive_allyrs=. then secuob_r_win99positive_allyrs=0; run; 


*take positive vieall_exvieall_r and winsorize it at 1% and 99% level for the entire sample period;
data b3; set ss4;
	if vieall_exvieall_r gt 0; 
	keep rid yr vieall_exvieall_r;
	run;
proc sort data=b3 out=b4 nodupkey; by rid yr; run;

data b4; set b4;
	vieall_win99=vieall_exvieall_r; run;
	
%wins_bynone(dsetin=b4, dsetout=b5, byvar=none, vars=vieall_win99, type=winsor, pctl=1 99);


/*get vieall_win99*/
proc sql; create table ss4 
	as select a.*, b.vieall_win99 
	from ss4 as a 
	left join b5 as b 
	on a.rid=b.rid and a.yr=b.yr; quit; *6034;

data ss4; set ss4;
	if vieall_win99=. then vieall_win99=0; run; 


*keep only obs in sec.LARS_srs_cfd95_0514_sus_june02 that find matches in ss4;
proc sql; create table LARS_h4 
	as select a.*, b.yr from LARS_h2 as a, ss4 as b 
	where a.rssdhh=b.rid 
	and a.year=b.yr; quit; 


data LARS_h4; set LARS_h4;
	log_income=log(income);
	log_amount=log(amount);
	drop year;
run; *4881313;

*winsorize loan-level variables;
%let var_wins=dti log_income log_amount dti;
%wins(LARS_h4,99,1,yr);  

/*get bank-year Y9C variables from ss4*/
proc sql; create table LARS_h5 
	as select a.*, b.*
	from LARS_h4 as a, ss4 as b 
	where a.rssdhh=b.rid 
	and a.yr=b.yr; quit; 


data LARS_h5; set LARS_h5;
	if hoepa=1 then hoepa_dummy=1;
	if hoepa=2 then hoepa_dummy=0; 
	if lien=3 then not_secure=1; else not_secure=0; run;

/*winsorization*/
%let var_wins=log_income dti log_amount;
%wins(LARS_h5,99,1,yr); 

data LARS_h5 
	(rename=(
	vieall_win99=Consolidated_VIE_Share 
	secuob_r_win99posit_allyrs=OffBS_Securitization 
	stress=Stress_Test 
	lsec_ta_r=Securities_Share 
	lfiloan=Loan_Income_Share 
	lploanr2=NPL 
	lcdr=Core_Deposit_Share 
	ldeposit_rate=Deposit_Rate 
	ch_wsf_rta=WSF_Growth 
	lroa_crisis_loss2=Writedowns 
	lroa_ex_crisis_loss2=ROA_ex_Writedowns 
	lt1rcapr2=Tier1_Capital_Ratio 
	lsize06=Bank_Size 
	dift1rcapr=Impact_On_Tier1_Capital_Ratio 
	sold_unaffi=sold 
	log_income=Log_Applicant_Income 
	dti=Loan_to_Income 
	log_amount=Log_Mortgage_Amount 
	hoepa_dummy=High_Cost
		));  
	set LARS_h5; run;

/*require abcp=0*/
data LARS_h5; set LARS_h5;
	if abcp=0; run;


data sec.d00_cfd95_hete_0514_50_jan2017; set LARS_h5; run; 

proc export data=sec.d00_cfd95_hete_0514_50_jan2017 outfile= "F:\bxie\d00_cfd95_hete_0514_50_jan2017.dta" replace;  
run; *This is the final loan-level dataset for 2005-2014 to run loan-level regressions to test the relation between Consolidated_VIE_Share and mortgage approval and sale;




