/*========================================================================================================
Winsorization by period
========================================================================================================*/
%macro wins(data,high,low,by); *define winsorized variables outside;

data &data;
   set &data;
   %let dim = %eval(%sysfunc(length(%sysfunc(compbl  (1&var_wins))))-
                  %sysfunc(length(%sysfunc(compress(1&var_wins))))+
                  %eval(%length(&var_wins)>0));

proc sort data=&data; by &by;

proc univariate data=&data noprint; by &by;
   var &var_wins;
   output out=outstat
   pctlpts=&low &high pctlpre=winsname1 - winsname&dim;
   run;

data &data;
   merge &data outstat; by &by;
   array a &var_wins;
   %macro assign(level,num_level);
      %do i=1 %to &dim;
         &level.&i=winsname&i.&num_level;
         drop winsname&i.&num_level;
         %end;
   %mend assign;
   %assign(high,&high);
   %assign(low,&low);
   array high high1-high&dim;
   array low low1-low&dim;
   do i=1 to &dim;
       if a[i]>high[i] then a[i]=high[i];
       if a[i] ne . and a[i]<low[i] then a[i]=low[i];
       end;
drop high1-high&dim low1-low&dim;

%mend wins;

/*example to invoke*/
%let var_wins= roa;
%wins(b1,99,1,yr); 

/*========================================================================================================*/



/*========================================================================================================
Winsorization by the entire sample period
========================================================================================================*/
%macro wins_bynone(dsetin=, dsetout=, byvar=none, vars=, type=winsor, pctl=1 99);
	%if &dsetout = %then %let dsetout = &dsetin;
	%let varL=;
	%let varH=;
	%let xn=1;
	%do %until ( %scan(&vars,&xn)= );
	%let token = %scan(&vars,&xn);
	%let varL = &varL &token.L;
	%let varH = &varH &token.H;
	%let xn=%EVAL(&xn + 1);
	%end;
	%let xn=%eval(&xn-1);
	data xtemp;
	set &dsetin;
	run;
	%if &byvar = none %then %do;
	data xtemp;
	set xtemp;
	xbyvar = 1;
	run;
	%let byvar = xbyvar;
	%end;
	proc sort data = xtemp;
	by &byvar;
	run;
	proc univariate data = xtemp noprint;
	by &byvar;
	var &vars;
	output out = xtemp_pctl PCTLPTS = &pctl PCTLPRE = &vars PCTLNAME = L H;
	run;
	data &dsetout;
	merge xtemp xtemp_pctl;
	by &byvar;
	array trimvars{&xn} &vars;
	array trimvarl{&xn} &varL;
	array trimvarh{&xn} &varH;
	do xi = 1 to dim(trimvars);
	%if &type = winsor %then %do;
	if not missing(trimvars{xi}) then do;
	if (trimvars{xi} < trimvarl{xi}) then trimvars{xi} = trimvarl{xi};
	if (trimvars{xi} > trimvarh{xi}) then trimvars{xi} = trimvarh{xi};
	end;
	%end;
	%else %do;
	if not missing(trimvars{xi}) then do;
	if (trimvars{xi} < trimvarl{xi}) then delete;
	if (trimvars{xi} > trimvarh{xi}) then delete;
	end;
	%end;
	end;
	drop &varL &varH xbyvar xi;
	run;
%mend wins_bynone;

/* example to invoke*/
%wins_bynone(dsetin=b1, dsetout=b2, byvar=none, vars=roa, type=winsor, pctl=1 99);
